# -*- coding: UTF-8 -*-
# !/usr/bin/env python

from kafka import KafkaConsumer
from libs.cache import redis_client
import logging
from linucb.views.linucb import LinUCB
import json
from trans2es.models.tag import TopicTag
import traceback

class KafkaManager(object):
    # kafka信息
    kafka_broker_list = "192.168.13.114:9092,192.168.13.116:9092,192.168.13.115:9092"
    topic_name = "alpha-maidian-data"
    consumser_obj = None

    @classmethod
    def get_kafka_consumer_ins(cls, topic_name=None):

        if not cls.consumser_obj:
            topic_name = cls.topic_name if not topic_name else topic_name
            cls.consumser_obj = KafkaConsumer(bootstrap_servers=cls.kafka_broker_list)
            cls.consumser_obj.subscribe([topic_name])

        return cls.consumser_obj

class CollectData(object):

    def __init__(self):
        self.linucb_matrix_redis_prefix = "physical:linucb:device_id:"
        self.linucb_recommend_redis_prefix = "physical:linucb:tag_recommend:device_id:"
        # 默认
        self.user_feature = [0,1]


    def _get_user_linucb_info(self, device_id):
        try:
            redis_key = self.linucb_matrix_redis_prefix + str(device_id)

            # dict的key为标签ID，value为4个矩阵
            redis_linucb_tag_data_dict = redis_client.hgetall(redis_key)

            return redis_linucb_tag_data_dict
        except:
            logging.error("catch exception,err_msg:%s" % traceback.format_exc())
            return dict()

    def update_recommend_tag_list(self, device_id,user_feature=None):
        try:
            recommend_tag_list = list()
            redis_linucb_tag_data_dict = self._get_user_linucb_info(device_id)
            if len(redis_linucb_tag_data_dict) == 0:
                recommend_tag_list = LinUCB.get_default_tag_list()
                LinUCB.init_device_id_linucb_info(redis_client, self.linucb_matrix_redis_prefix,device_id,recommend_tag_list)
            else:
                user_feature = user_feature if user_feature else self.user_feature
                recommend_tag_list = LinUCB.linucb_recommend_tag(redis_linucb_tag_data_dict,user_feature,redis_linucb_tag_data_dict.keys())

            if len(recommend_tag_list) > 0:
                tag_recommend_redis_key = self.linucb_recommend_redis_prefix + str(device_id)
                redis_client.set(tag_recommend_redis_key, recommend_tag_list)
                # Todo:设置过期时间，调研set是否支持
                redis_client.expire(tag_recommend_redis_key, 7*24*60*60)

            return True
        except:
            logging.error("catch exception,err_msg:%s" % traceback.format_exc())
            return False

    def update_user_linucb_tag_info(self, reward, device_id, tag_id, user_feature=None):
        try:
            user_feature = user_feature if user_feature else self.user_feature
            return LinUCB.update_linucb_info(user_feature, reward, tag_id, device_id,self.linucb_matrix_redis_prefix,redis_client)
        except:
            logging.error("update_user_linucb_tag_info error!")
            return False


    def consume_data_from_kafka(self,topic_name=None):
        try:
            kafka_consumer_obj = KafkaManager.get_kafka_consumer_ins(topic_name)
            for ori_msg in kafka_consumer_obj:
                logging.info(ori_msg)

                value_dict = json.loads(ori_msg.value)
                if "@raw" in value_dict and "type" in value_dict["@raw"] and "on_click_feed_topic_card"==value_dict["@raw"]["type"]:
                    topic_id = value_dict["@raw"]["params"]["business_id"]
                    device_id = value_dict["@raw"]["device"]["device_id"]

                    logging.info("consume topic_id:%s,device_id:%s" % (str(topic_id),str()))

                    tag_list = list()
                    sql_query_results = TopicTag.objects.filter(is_online=True,topic_id=topic_id)
                    for sql_item in sql_query_results:
                        tag_list.append(sql_item.tag_id)

                    is_click = 1
                    is_vote = 0

                    user_feature = [is_click, is_vote]
                    reward = 1 if is_click or is_vote else 0

                    for tag_id in tag_list:
                        self.update_user_linucb_tag_info(reward,device_id,tag_id,user_feature)

                    # 更新该用户的推荐tag数据，放在 更新完成user tag行为信息之后
                    self.update_recommend_tag_list(device_id, user_feature)

            return True
        except:
            logging.error("catch exception,err_msg:%s" % traceback.format_exc())
            return False
